#!/usr/bin/perl

# 参考資料: http://monitor.eduroam.org/database.php

# Created by Hajime Shimada (Advanced Networking Division, Information Technology Center, Nagoya University.)
# Ver. 0.1 August 25, 2015
# Ver. 0.2 September 9, 2015
# Ver. 0.3 October 7, 2015

#
# 組織情報 (適時編集すること)
#
$country = "jp"; # 国
$type = "3"; # 学術機関?
$realm = "fixit.ac.jp"; # レルム名 (代理認証システム利用では example.eduroam.jp)
# (仮名アカウント発行システム利用では、upki.eduroam.jp ではなく、ダミーとして大学のドメイン名を書くこと(下記参照)) 
$org_name_en = "fixit University"; # 機関の名称(英字表記)
$org_name_ja = "fixit大学"; # 機関の名称(日本語表記)
# 機関の本部所在(英字表記)
$address_street_org = "fixit-cho, fixit-ku"; # 区以下
$address_city_org = "fixit(Nagoya), fixit(Aichi)"; # 市, 県
# 機関のeduroam運用に関する連絡先(spam対策でnone推奨)
$contact_name = "none";
$contact_email = "none";
$contact_phone = "none";
# eduroamに関する情報提供サイトのURLを記述(ブランク推奨)
$info_url_en = "";
$info_url_ja = "";
$policy_url_en = "";


#
# スクリプト本体
#

use utf8;
use Encode qw(from_to encode decode);
use Text::CSV_XS;

# ファイル入出力確認

if(@ARGV != 2){
    die "usage: eduroam_APinfo_generator.pl input.csv output.xml\n";
}

$in_file_name = $ARGV[0];
$out_file_name = $ARGV[1];

if((!open(IN_FILE, "$in_file_name"))){
    die "Can't open $in_file_name\n";
}
if((!open(OUT_FILE, ">$out_file_name"))){
    die "Can't open $out_file_name\n";
}


# 日付情報

($day, $mon, $year) = (localtime(time))[3..5];
$year = 1900 + $year;
$mon = sprintf("%02d", $mon + 1);
$day = sprintf("%02d", $day);
$time_stamp = ${year} . "-" . ${mon} . "-" . ${day};


# CSV入力

$cnt = 0;
$line_no = 0;
$csv = Text::CSV_XS->new({'binary' => 1});

while(<IN_FILE>){
    if($csv->parse($_)){
	@line = $csv->fields();
    }else{
	$err = $csv->error_input();
	die "Cannot parse " . $err . " in CSV file.\n";
    }

    for($i = 2; $i < 9; $i++){
	$line[$i] =~ s/&/&amp;/g;
    }

    if($line[0] =~ /^country/){
	$country = $line[2];
    }
    if($line[0] =~ /^type/){
	$type = $line[2];
    }
    if($line[0] =~ /^realm/){
	$realm = $line[2];
    }
    if($line[0] =~ /^org_name_ja/){
	$org_name_ja = $line[2];
	from_to($org_name_ja, 'sjis', 'utf8');
    }
    if($line[0] =~ /^org_name_en/){
	$org_name_en = $line[2];
    }
    if($line[0] =~ /^city/){
	$address_city_org = $line[2];
    }
    if($line[0] =~ /^street/){
	$address_street_org = $line[2];
    }
    if($line[0] =~ /^contact_name/){
	$contact_name = $line[2];
	from_to($contact_name, 'sjis', 'utf8');
    }
    if($line[0] =~ /^contact_email/){
	$contact_email = $line[2];
    }
    if($line[0] =~ /^contact_phone/){
	$contact_phone = $line[2];
    }
    if($line[0] =~ /^infoURLja/){
	$info_url_ja = $line[2];
	from_to($info_url_ja, 'sjis', 'utf8');
    }
    if($line[0] =~ /^infoURLen/){
	$info_url_en = $line[2];
    }
    if($line[0] =~ /^policyURL/){
	$policy_url_en = $line[2];
    }
    if($line[0] =~ /^time_stamp/){
	if($line[2] =~ /^[0-9]+-[0-9]+-[0-9]+$/){
	    $time_stamp = $line[2];
	}
    }
  
    if(($line[0] =~ /[0-9]+\.[0-9]+/) &&
       ($line[1] =~ /[0-9]+\.[0-9]+/)){ # 有効な経度緯度か?
	$line[0] =~ /([0-9]+\.[0-9]+)/;
	$longitude[$cnt] = $1;
	$line[1] =~ /([0-9]+\.[0-9]+)/;
	$latitude[$cnt] = $1;
	$building_en[$cnt] = $line[2];
	$building_ja[$cnt] = $line[3];
	$address_street[$cnt]= $line[4];
	$address_city[$cnt]= $line[5];
	$ssid[$cnt]= $line[6];
	$encryption[$cnt]= $line[7];
	$port_restrict[$cnt]= $line[8];

	from_to($building_ja[$cnt], 'sjis', 'utf8');

	$cnt++;
    }

    $line_no++;
}

close IN_FILE;

#debug
#print $line_no . "\n";
#print $cnt . "\n";
#for($i = 0; $i < $cnt; $i++){
#    print "$longitude[$i], $latitude[$i], $building_en[$i], $building_ja[$i], $address_street[$i], $address_city[$i], $ssid[$i], $encryption[$i], $num_of_AP[$i]\n";
#}
#die;

# XML出力

print OUT_FILE <<EOF;
<?xml version="1.0" encoding="UTF-8"?>

<institutions>

  <institution>
EOF
    
print OUT_FILE "    <country>$country</country>\n";
print OUT_FILE "    <type>$type</type>\n\n";
print OUT_FILE "    <inst_realm>$realm</inst_realm>\n\n";
print OUT_FILE "      <org_name lang=\"en\">$org_name_en</org_name>\n";
print OUT_FILE "      <org_name lang=\"ja\">$org_name_ja</org_name>\n\n";
print OUT_FILE "      <address>\n";
print OUT_FILE "        <street>$address_street_org</street>\n";
print OUT_FILE "        <city>$address_city_org</city>\n";
print OUT_FILE "      </address>\n\n";

print OUT_FILE "      <contact>\n";
print OUT_FILE "        <name>$contact_name</name>\n";
print OUT_FILE "        <email>$contact_email</email>\n";
print OUT_FILE "        <phone>$contact_phone</phone>\n";
print OUT_FILE "      </contact>\n\n";

print OUT_FILE "    <info_URL lang=\"en\">$info_url_en</info_URL>\n";
print OUT_FILE "    <info_URL lang=\"ja\">$info_url_ja</info_URL>\n";
print OUT_FILE "    <policy_URL lang=\"en\">$policy_url_en</policy_URL>\n\n";

print OUT_FILE "    <ts>${time_stamp}T00:00:00+00:00</ts>\n\n\n";

for($i = 0; $i < $cnt; $i++){
    print OUT_FILE "    <location>\n\n";
    print OUT_FILE "      <longitude>$longitude[$i]</longitude>\n";
    print OUT_FILE "      <latitude>$latitude[$i]</latitude>\n";
    print OUT_FILE "      <loc_name lang=\"en\">$building_en[$i]</loc_name>\n";
    print OUT_FILE "      <loc_name lang=\"ja\">$building_ja[$i]</loc_name>\n";
    print OUT_FILE "      <address>\n";
    print OUT_FILE "        <street>$address_street[$i]</street>\n";
    print OUT_FILE "        <city>$address_city[$i]</city>\n";
    print OUT_FILE "      </address>\n\n";
    print OUT_FILE "      <SSID>$ssid[$i]</SSID>\n";
    print OUT_FILE "      <enc_level>$encryption[$i]</enc_level>\n\n";
    print OUT_FILE "      <port_restrict>$port_restrict[$i]</port_restrict>\n";
    print OUT_FILE "    </location>\n\n";
}

print OUT_FILE "  </institution>\n\n";
print OUT_FILE "</institutions>\n\n";

close OUT_FILE;
