#!/usr/bin/perl

# 参考資料: http://monitor.eduroam.org/database.php

# Created by Hajime Shimada (Advanced Networking Division, Information Technology Center, Nagoya University.)
# Ver. 0.1 October 7, 2015

# super ad-hoc


#
# スクリプト本体
#

use utf8;
use Encode qw(from_to encode decode);
use Text::CSV_XS;


# ファイル入出力確認

if(@ARGV != 2){
    die "usage: eduroam_APinfo2csv.pl input.xml output.csv\n";
}

$in_file_name = $ARGV[0];
$out_file_name = $ARGV[1];

if((!open(IN_FILE, "$in_file_name"))){
    die "Can't open $in_file_name\n";
}
if((!open(OUT_FILE, ">$out_file_name"))){
    die "Can't open $out_file_name\n";
}

#
# XML読み込み
#

$APinfo = 0;
$APnum = 0;

while(<IN_FILE>){
    if($APinfo == 0){
	if($_ =~ /<country>(.*)<\/country>/){
	    $country = $1;
	}
	if($_ =~ /<type>(.*)<\/type>/){
	    $type = $1;
	}
	if($_ =~ /<inst_realm>(.*)<\/inst_realm>/){
	    $realm = $1;
	}
	if($_ =~ /<org_name\ lang=\"en\">(.*)<\/org_name>/){
	    $org_name_en = $1;
	}
	if($_ =~ /<org_name\ lang=\"ja\">(.*)<\/org_name>/){
	    $org_name_ja = $1;
	}
	if($_ =~ /<street>(.*)<\/street>/){
	    $street = $1;
	}
	if($_ =~ /<city>(.*)<\/city>/){
	    $city = $1;
	}
	if($_ =~ /<name>(.*)<\/name>/){
	    $contact_name = $1;
	}
	if($_ =~ /<email>(.*)<\/email>/){
	    $contact_email = $1;
	}
	if($_ =~ /<phone>(.*)<\/phone>/){
	    $contact_phone = $1;
	}
	if($_ =~ /<info_URL\ lang=\"en\">(.*)<\/info_URL>/){
	    $infoURLen = $1;
	}
	if($_ =~ /<info_URL\ lang=\"ja\">(.*)<\/info_URL>/){
	    $infoURLja = $1;
	}
	if($_ =~ /<policy_URL\ lang=\"en\">(.*)<\/policy_URL>/){
	    $policyURL = $1;
	}
	if($_ =~ /<location>/){
	    $APinfo = 1;
	}
    }else{
	if($_ =~ /<longitude>(.*)<\/longitude>/){
	    $longitude[$APnum] = $1;
	}
	if($_ =~ /<latitude>(.*)<\/latitude>/){
	    $latitude[$APnum] = $1;
	}
	if($_ =~ /<loc_name\ lang=\"en\">(.*)<\/loc_name>/){
	    $loc_name_en[$APnum] = $1;
	}
	if($_ =~ /<loc_name\ lang=\"ja\">(.*)<\/loc_name>/){
	    $loc_name_ja[$APnum] = $1;
	}
	if($_ =~ /<street>(.*)<\/street>/){
	    $APstreet[$APnum] = $1;
	}
	if($_ =~ /<city>(.*)<\/city>/){
	    $APcity[$APnum] = $1;
	}
	if($_ =~ /<SSID>(.*)<\/SSID>/){
	    $SSID[$APnum] = $1;
	}
	if($_ =~ /<enc_level>(.*)<\/enc_level>/){
	    $enc_level[$APnum] = $1;
	}
	if($_ =~ /<port_restrict>(.*)<\/port_restrict>/){
	    $port_restrict[$APnum] = $1;
	}
	if($_ =~ /<\/location>/){
	    $APinfo = 0;
	    $APnum++;
	}

    }
}


# CSV出力

$csv = Text::CSV_XS->new({'binary' => 1});

$warning_word = encode(sjis, "\n\n次の部分をExcelファイルの「●組織情報設定」の所にコピーして下さい。\n\n");
print OUT_FILE $warning_word;

@line_in = ("country", "", $country);
$status = $csv->combine(@line_in);
$line_out = $csv->string();
from_to($line_out, 'utf8', 'sjis');
print OUT_FILE $line_out . "\n";

@line_in = ("type", "", $type);
$status = $csv->combine(@line_in);
$line_out = $csv->string();
from_to($line_out, 'utf8', 'sjis');
print OUT_FILE $line_out . "\n";

@line_in = ("realm", "", $realm);
$status = $csv->combine(@line_in);
$line_out = $csv->string();
from_to($line_out, 'utf8', 'sjis');
print OUT_FILE $line_out . "\n";

@line_in = ("org_name_en", "", $org_name_en);
$status = $csv->combine(@line_in);
$line_out = $csv->string();
from_to($line_out, 'utf8', 'sjis');
print OUT_FILE $line_out . "\n";

@line_in = ("org_name_ja", "", $org_name_ja);
$status = $csv->combine(@line_in);
$line_out = $csv->string();
from_to($line_out, 'utf8', 'sjis');
print OUT_FILE $line_out . "\n";

@line_in = ("street", "", $street);
$status = $csv->combine(@line_in);
$line_out = $csv->string();
from_to($line_out, 'utf8', 'sjis');
print OUT_FILE $line_out . "\n";

@line_in = ("city", "", $city);
$status = $csv->combine(@line_in);
$line_out = $csv->string();
from_to($line_out, 'utf8', 'sjis');
print OUT_FILE $line_out . "\n";

@line_in = ("contact_name", "", $contact_name);
$status = $csv->combine(@line_in);
$line_out = $csv->string();
from_to($line_out, 'utf8', 'sjis');
print OUT_FILE $line_out . "\n";

@line_in = ("contact_email", "", $contact_email);
$status = $csv->combine(@line_in);
$line_out = $csv->string();
from_to($line_out, 'utf8', 'sjis');
print OUT_FILE $line_out . "\n";

@line_in = ("contact_phone", "", $contact_phone);
$status = $csv->combine(@line_in);
$line_out = $csv->string();
from_to($line_out, 'utf8', 'sjis');
print OUT_FILE $line_out . "\n";

@line_in = ("infoURLen", "", $infoURLen);
$status = $csv->combine(@line_in);
$line_out = $csv->string();
from_to($line_out, 'utf8', 'sjis');
print OUT_FILE $line_out . "\n";

@line_in = ("infoURLja", "", $infoURLja);
$status = $csv->combine(@line_in);
$line_out = $csv->string();
from_to($line_out, 'utf8', 'sjis');
print OUT_FILE $line_out . "\n";

@line_in = ("policyURL", "", $policyURL);
$status = $csv->combine(@line_in);
$line_out = $csv->string();
from_to($line_out, 'utf8', 'sjis');
print OUT_FILE $line_out . "\n";

$warning_word = encode(sjis, "\n\n次の部分をExcelファイルの「●AP情報設定」の所にコピーして下さい。\n\n");
print OUT_FILE $warning_word;

for($i = 0; $i < $APnum; $i++){
    @line_in = ($longitude[$i], $latitude[$i], $loc_name_en[$i], $loc_name_ja[$i], $APstreet[$i], $APcity[$i], $SSID[$i], $enc_level[$i], $port_restrict[$i]);
    $status = $csv->combine(@line_in);
    $line_out = $csv->string();
    $line_out =~ s/&amp;/&/g;
    from_to($line_out, 'utf8', 'sjis');
    print OUT_FILE $line_out . "\n";
}
